// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import TIM
import UserSettingsServices
import MaternityServices
import MaternityData
import ErrorData
import Combine

final class UserContext : UserContextProtocol {
    @DI private var timAuth: TIMAuth
    @DI private var timStorage: TIMDataStorage
    @DI private var userSettings: UserSettingsService
    @DI private var logger: Logger
    @DI private var dataCardService: DataCardService

    /// DataCard objects for selected maternity. This should always be fetched on log in
    private (set) var maternities: [DataCard]?

    /// The selected maternity - the user should never be able to use the app without this property being set.
    var selectedMaternity: DataCard? {
        didSet {
            guard let id = selectedMaternity?.id else {
                logger.critical("The selected maternity object should never be nulled!")
                return
            }
            userSettings.save(id, for: .selectedEpisodeOfCareId)

            if oldValue?.id != selectedMaternity?.id {
                logger.debug("Changed selected maternity:  \(selectedMaternity?.id ?? "-")")
            }
            #warning("We should probably raise some kind of notification here to let all views know, that the selected maternity changed!")
        }
    }

    var hasHandledOnboardingFlow: Bool {
        userSettings.get(key: .hasHandledOnboarding) ?? false
    }

    var isAuthenticated: Bool {
        timAuth.isLoggedIn
    }

    var userId: String? {
        timAuth.refreshToken?.userId
    }

    var hasUserRegistered: Bool {
        !timStorage.availableUserIds.isEmpty
    }

    func fetchAllInitialData() -> AnyPublisher<Void, ServiceError> {
        dataCardService.fetchDataCards()
            .handleEvents(receiveOutput: { [weak self] (cards: [DataCard]) in
                if let id = self?.userSettings.get(key: .selectedEpisodeOfCareId) {
                    self?.selectedMaternity = cards.first(where: { $0.episodeOfCareIdentifier == id })
                }
                if self?.selectedMaternity == nil {
                    self?.selectedMaternity = cards.mostRecentDataCard()
                }
            })
            .mapToVoid()
            .eraseToAnyPublisher()
    }

    func logout() {
        guard let userId = userId else {
            resetForUndefinedUserState()
            return
        }

        timAuth.logout()
        NotificationCenter.default.post(
            notification: Notification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.logout(userId: userId)
        )
    }

    func resetAllUserData() {
        if let userId = userId {
            timAuth.logout()
            timStorage.clear(userId: userId)
            setHasHandledOnboardingFlow(!timStorage.availableUserIds.isEmpty)
            userSettings.remove(key: .selectedEpisodeOfCareId)
        } else {
            setHasHandledOnboardingFlow(false)
        }
        resetToOnboarding()
    }

    func setHasHandledOnboardingFlow(_ handled: Bool) {
        userSettings.save(handled, for: .hasHandledOnboarding)
    }

    private func resetForUndefinedUserState() {
        logger.critical("Tried to logout, but did not have a userId. Resetting all registered users and resetting to onboarding!")
        userSettings.remove(key: .selectedEpisodeOfCareId)
        timStorage.clearAllUsers()
        setHasHandledOnboardingFlow(false)
        resetToOnboarding()
    }

    private func resetToOnboarding() {
        NotificationCenter.default.post(
            notification: Notification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.resetToOnboarding
        )
    }
}
