// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import CMSData
import ErrorData
import CMSServices
import ChecklistValueServices
import Combine

/// This service is merger between the checklist values and the checklist content, which are placed on two different backends.
/// This service will use the two different services and merge it to one result.
final class ChecklistServiceImpl : ChecklistService {
    @DI private var checklistValueService: ChecklistValueService
    @DI private var checklistContentService: ChecklistContentService
    @DI private var userContext: UserContextProtocol
    @DI private var logger: Logger

    func fetchChecklists() -> AnyPublisher<[Checklist], ServiceError> {
        guard let maternityId = userContext.selectedMaternity?.id else {
            logger.critical("Tried to fetch checklists without a selected maternity!")
            return Fail(error: ServiceError.unexpectedError(nil)).eraseToAnyPublisher()
        }
        return checklistContentService
            .fetchChecklistContent()
            .zip(checklistValueService.fetchChecklists(maternityId: maternityId))
            .map({ (zip: (content: [ChecklistContent], values: [String])) -> [Checklist] in
                zip.content.map { (content: ChecklistContent) -> Checklist in
                    Checklist.create(from: content, checkedIds: zip.values)
                }
            })
            .eraseToAnyPublisher()
    }

    func updateChecklistItem(itemId: String, isChecked: Bool) -> AnyPublisher<Void, ServiceError> {
        guard let maternityId = userContext.selectedMaternity?.id else {
            logger.critical("Tried to update checklist items without a selected maternity!")
            return Fail(error: ServiceError.unexpectedError(nil)).eraseToAnyPublisher()
        }
        return checklistValueService
            .updateChecklistItem(maternityId: maternityId, itemId: itemId, isChecked: isChecked)
            .eraseToAnyPublisher()
    }
}
