//
//  HealthcareContactsServiceImpl.swift
//  MinGraviditet
//
//  Created by Kim de Vos on 08/01/2021.
//

import Foundation
import Combine
import ErrorData
import MaternityData
import MaternityServices
import CMSServices

final class HealthcareContactsServiceImpl: HealthcareContactsService {
    @DI private var userContext: UserContextProtocol
    @DI private var dictionaryService: DictionaryService
    
    func fetchGeneralPractitioner() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let generalPractitioner = userContext.selectedMaternity?.personalDataSection?.generalPractitioner else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_gp_name(),
                        primaryValue: generalPractitioner.name,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_gp_address(),
                        primaryValue: generalPractitioner.address?.streetAndNumber
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_gp_postal_code(),
                        primaryValue: generalPractitioner.address?.zip
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_gp_city(),
                        primaryValue: generalPractitioner.address?.city
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchMidWife() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let midwifeClinic = userContext.selectedMaternity?.midwifeClinicSection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_midwife_assigned_clinic(),
                        primaryValue: midwifeClinic.assignedMidwifeClinic?.name,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_midwife_craft_class_desired(),
                        primaryValue: midwifeClinic.parentalCraftClassDesired.description,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_midwife_consultation_form(),
                        primaryValue: midwifeClinic.consultationForm,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_midwife_clinic_wishes(),
                        primaryValue: midwifeClinic.midwifeConsultationWishes?.location,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_midwife_practitioner_wishes(),
                        primaryValue: midwifeClinic.midwifeConsultationWishes?.midwife.name.fullName,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchBirthplace() -> AnyPublisher<[KeyValueModel], ServiceError> {
        guard let birthPlaceSection = userContext.selectedMaternity?.birthPlaceSection else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_birthplace_planned(),
                        primaryValue: birthPlaceSection.plannedBirthPlace,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_birthplace_revised(),
                        primaryValue: birthPlaceSection.revisedBirthPlace,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: R.string.localizable.journal_section_health_care_birthplace_desired(),
                        primaryValue: birthPlaceSection.desiredBirthPlace,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchNurse() -> AnyPublisher<[KeyValueModel], ServiceError> {

        guard let healthNurse = userContext.selectedMaternity?.healthCareSection?.healthNurse else {
            return Just([])
                .mapToServiceError()
                .eraseToAnyPublisher()
        }

        return Just([
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_health_care_nurse_ward(),
                primaryValue: userContext.selectedMaternity?.healthCareSection?.ward?.name
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_health_care_nurse_nurse_name(),
                primaryValue: healthNurse.name
            ),
            
            KeyValueModel.dictionaryModel(
                title: R.string.localizable.journal_section_health_care_nurse_nurse_phonenumber(),
                primaryValue: healthNurse.phone
            )
        ])
        .mapToServiceError()
        .eraseToAnyPublisher()
    }
}
