// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import WebKit

/// View for showing HTML.
///
struct HTMLView: UIViewRepresentable {
    /// The height will be published when the HTML is loaded
    @Binding var dynamicHeight: CGFloat

    /// Will return `false` when html is loaded
    @Binding var isLoading: Bool

    /// The file path to the HTML file
    let htmlFilePath: URL

    /// The HTML content that need to be replaced
    let html: String
    
    func makeUIView(context: Context) -> WKWebView {
        let webView = WKWebView()

        webView.scrollView.isScrollEnabled = false
        webView.scrollView.alwaysBounceVertical = false
        webView.navigationDelegate = context.coordinator
        webView.backgroundColor = .clear

        return webView
    }

    func updateUIView(_ uiView: WKWebView, context: Context) {
        if let fileTemplate = try? String(contentsOf: htmlFilePath, encoding: .utf8) {
            let html = fileTemplate.replacingOccurrences(of: "[CONTENT]", with: self.html)
            uiView.loadHTMLString(html, baseURL: htmlFilePath.deletingLastPathComponent())
        }
    }

    func makeCoordinator() -> Coordinator { Coordinator(self) }

    final class Coordinator: NSObject, WKNavigationDelegate {
        @DI private var logger: Logger
        private let parent: HTMLView

        private static let fileScheme: String = "file"

        /// The allowed URL schemes that are allowed to open
        let allowedNavigationSchemes: [String] = [ "https", "http", "tel", "mailto", fileScheme ]

        init(_ parent: HTMLView) {
            self.parent = parent
        }

        func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
            webView.evaluateJavaScript("document.documentElement.scrollHeight", completionHandler: { height, _ in
                DispatchQueue.main.async {
                    self.parent.dynamicHeight = height as? CGFloat ?? 0
                    self.parent.isLoading = false
                }
            })
        }

        func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
            // Check if the scheme is allowed, and then open it
            if let url = navigationAction.request.url, let scheme = url.scheme {
                if scheme == Self.fileScheme && url.absoluteString.contains(Bundle.main.bundleURL.absoluteString) {
                    decisionHandler(.allow) // Allow to load local content from the app
                } else {
                    decisionHandler(.cancel) // Always cancel to avoid navigation inside the embedded web view

                    if allowedNavigationSchemes.contains(scheme) {
                        UIApplication.shared.openSafe(url: url, options: nil, completionHandler: nil)
                    } else {
                        // Raising an error at this point, since something might be wrong on the CMS if this occurs...
                        logger.error("Cancelled web view navigation in `HTMLView` for disallowed scheme: \"\(scheme)\"\(scheme == Self.fileScheme ? " (FILE SCHEME NOT IN APP BUNDLE)" : "")")
                    }
                }
            } else {
                decisionHandler(.cancel) // Cancel if there is no URL
            }
        }
    }
}
